@@ -15,7 +15,7 @@ import { formatTime } from '@grafana/ui/src/components/uPlot/config/UPlotAxisBui
 import { StackingGroup, preparePlotData2 } from '@grafana/ui/src/components/uPlot/utils';
 
 import { distribute, SPACE_BETWEEN } from './distribute';
-import { findRect, intersects, pointWithin, Quadtree, Rect } from './quadtree';
+import { intersects, pointWithin, Quadtree, Rect } from './quadtree';
 
 const groupDistr = SPACE_BETWEEN;
 const barDistr = SPACE_BETWEEN;
@@ -48,10 +48,10 @@ export interface BarsOptions {
   showValue: VisibilityMode;
   stacking: StackingMode;
   rawValue: (seriesIdx: number, valueIdx: number) => number | null;
-  getColor?: (seriesIdx: number, valueIdx: number, value: any) => string | null;
+  getColor?: (seriesIdx: number, valueIdx: number, value: unknown) => string | null;
   fillOpacity?: number;
-  formatValue: (seriesIdx: number, value: any) => string;
-  formatShortValue: (seriesIdx: number, value: any) => string;
+  formatValue: (seriesIdx: number, value: unknown) => string;
+  formatShortValue: (seriesIdx: number, value: unknown) => string;
   timeZone?: TimeZone;
   text?: VizTextDisplayOptions;
   onHover?: (seriesIdx: number, valueIdx: number) => void;
@@ -289,11 +289,11 @@ export function getConfig(opts: BarsOptions, theme: GrafanaTheme2) {
     radius: pctStacked
       ? 0
       : !isStacked
-      ? barRadius
-      : (u: uPlot, seriesIdx: number) => {
-          let isTopmostSeries = seriesIdx === u.data.length - 1;
-          return isTopmostSeries ? [barRadius, 0] : [0, 0];
-        },
+        ? barRadius
+        : (u: uPlot, seriesIdx: number) => {
+            let isTopmostSeries = seriesIdx === u.data.length - 1;
+            return isTopmostSeries ? [barRadius, 0] : [0, 0];
+          },
     disp: {
       x0: {
         unit: 2,
@@ -445,12 +445,11 @@ export function getConfig(opts: BarsOptions, theme: GrafanaTheme2) {
   const init = (u: uPlot) => {
     let over = u.over;
     over.style.overflow = 'hidden';
-    u.root.querySelectorAll('.u-cursor-pt').forEach((el) => {
-      if (el instanceof HTMLElement) {
-        el.style.borderRadius = '0';
-        if (opts.fullHighlight) {
-          el.style.zIndex = '-1';
-        }
+    u.root.querySelectorAll<HTMLDivElement>('.u-cursor-pt').forEach((el) => {
+      el.style.borderRadius = '0';
+
+      if (opts.fullHighlight) {
+        el.style.zIndex = '-1';
       }
     });
   };
@@ -471,14 +470,7 @@ export function getConfig(opts: BarsOptions, theme: GrafanaTheme2) {
 
         qt.get(cx, cy, 1, 1, (o) => {
           if (pointWithin(cx, cy, o.x, o.y, o.x + o.w, o.y + o.h)) {
-            if (isStacked) {
-              // choose the smallest hovered rect (when stacked bigger ones overlap smaller ones)
-              if (hRect == null || o.h * o.w < hRect.h * hRect.w) {
-                hRect = o;
-              }
-            } else {
-              hRect = o;
-            }
+            hRect = o;
           }
         });
       }
@@ -490,23 +482,11 @@ export function getConfig(opts: BarsOptions, theme: GrafanaTheme2) {
       bbox: (u, seriesIdx) => {
         let isHovered = hRect && seriesIdx === hRect.sidx;
 
-        let heightReduce = 0;
-        let widthReduce = 0;
-
-        // get height of bar rect at same index of the series below the hovered one
-        if (isStacked && isHovered && hRect!.sidx > 1) {
-          if (isXHorizontal) {
-            heightReduce = findRect(qt, hRect!.sidx - 1, hRect!.didx)!.h;
-          } else {
-            widthReduce = findRect(qt, hRect!.sidx - 1, hRect!.didx)!.w;
-          }
-        }
-
         return {
-          left: isHovered ? (hRect!.x + widthReduce) / uPlot.pxRatio : -10,
+          left: isHovered ? hRect!.x / uPlot.pxRatio : -10,
           top: isHovered ? hRect!.y / uPlot.pxRatio : -10,
-          width: isHovered ? (hRect!.w - widthReduce) / uPlot.pxRatio : 0,
-          height: isHovered ? (hRect!.h - heightReduce) / uPlot.pxRatio : 0,
+          width: isHovered ? hRect!.w / uPlot.pxRatio : 0,
+          height: isHovered ? hRect!.h / uPlot.pxRatio : 0,
         };
       },
     },
@@ -524,10 +504,9 @@ export function getConfig(opts: BarsOptions, theme: GrafanaTheme2) {
     });
 
     if (isStacked) {
-      //barsPctLayout = [null as any].concat(distrOne(u.data.length - 1, u.data[0].length));
-      barsPctLayout = [null as any].concat(distrOne(u.data[0].length, u.data.length - 1));
+      barsPctLayout = [null, ...distrOne(u.data[0].length, u.data.length - 1)];
     } else {
-      barsPctLayout = [null as any].concat(distrTwo(u.data[0].length, u.data.length - 1));
+      barsPctLayout = [null, ...distrTwo(u.data[0].length, u.data.length - 1)];
     }
 
     if (useMappedColors) {
